/***************************************************************************
                          connect.c  -  description
                             -------------------
    email                : diceman@dircon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "relay.h"


long getip(char *host) {

	long ia;
	struct hostent *h;

	if(o.debug) {
		fprintf(stderr, "Getting IP\n");
	}


	if((ia = inet_addr(host)) == -1) {
		if((h=gethostbyname(host)) == NULL) {
			if(o.debug) {
				fprintf(stderr, "Couldn't resolv ip\n");
			}
			return(-1);
		}	
		memcpy(&ia,h->h_addr,4);
	}
	
	if(o.debug) {
		fprintf(stderr, "Got ip ok\n");
	}
	return(ia);

}

int tcpconnect(long ipa) {

	struct sockaddr_in their_addr;
	int sockfd;
	int socket_f = 0;

	while((sockfd=socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		if(socket_f>12) {
			fprintf(stderr, "Still unable to create socket after 60 secs.... Try reducing the number of threads..... Exiting\n\n");
			exit(1);
		}
		socket_f++;
		if(o.verbose) {
			fprintf(stderr, "Unable to create socket.... sleeping for 5 seconds\n");
		}
		sleep(5);
	}
	
	their_addr.sin_family = AF_INET;
  their_addr.sin_port = htons(PORT);
  their_addr.sin_addr.s_addr = ipa;
  bzero(&(their_addr.sin_zero), 8);

	if((sockfd=connect_non_blocking(sockfd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr), o.c_timeout)) == -1) {
		if(o.verbose > 1){
			if(o.child) {
				fprintf(stderr,"! Unable to connect to host \n");
				fflush(stderr);
			}
			else {
				fprintf(stderr, "! Unable to connect to host\n");
			}
		}
		return(-1);
	}
	return(sockfd);
}


int connect_non_blocking(int sockfd, const struct sockaddr *thaddr, socklen_t salen, int nsec)
{

	int n, flags, error;
	socklen_t len;
	fd_set rset, wset;
	struct timeval tval;
	
	if((flags=fcntl(sockfd, F_GETFL, 0)) == -1){
		perror("Cant set flags");
		o.cur_childs--;
		pthread_detach(pthread_self());
		pthread_exit(NULL);
	}	

	if(fcntl(sockfd, F_SETFL, flags | O_NONBLOCK) == -1) {
		perror("Cant set socket options");
		o.cur_childs--;
		pthread_detach(pthread_self());
		pthread_exit(NULL);
	}
	error = 0;
	if((n = connect(sockfd, thaddr, salen)) <0)
		if(errno != EINPROGRESS)
			return(-1);

	if(n == 0)
		goto done;

	FD_ZERO(&rset);
	FD_SET(sockfd, &rset);
	wset = rset;
	tval.tv_sec = nsec;
	tval.tv_usec = 500000;

	if((n = select(sockfd +1, &rset, &wset, NULL, nsec ? &tval : NULL)) == 0) {
		close(sockfd);
		errno = ETIMEDOUT;
		return(-1);
	}
	else if(n==-1) {
		perror("Select failed\n");
	}
	
	if(FD_ISSET(sockfd, &rset) || FD_ISSET(sockfd, &wset)) {
		len = sizeof(error);
		if(getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &error, &len) < 0)
			return(-1);
	}
	else {
		fprintf(stderr, "\n\nERROR\n\nSocket not set..... quiting\n\n");
		exit(1);
	}

  done:
	if(fcntl(sockfd, F_SETFL, flags) == -1) {
		perror("Can't restore the socket opts\n");
		return(-1);
	}

	if(error) {
		close(sockfd);
		errno = error;
		return(-1);
	}

	return(sockfd);
}

	



int read_timeo(int sockfd, char *string, int max, int timeo) {

	struct timeval tv;

	int numbytes = 0;

  fd_set readfds;
  
  tv.tv_sec = timeo;
  tv.tv_usec = 500000;


  FD_ZERO(&readfds);
  FD_SET(sockfd, &readfds);


  select(sockfd+1, &readfds, NULL, NULL, &tv);
 
  if(FD_ISSET(sockfd, &readfds)) {
        if((numbytes=read(sockfd, string, max)) < 1) {
					return(0);
				}
				else {
					string[numbytes] = '\0';
				}
  }
  else {
        return(-1);
  }
  return(1);

}

void sig_int(int signo) {
	

	switch(signo) {
		case SIGINT:
		fprintf(stderr, "\n\nCaught SIGINT..... Cleaning up!\n\n");
		exit(1);
		break;
		case SIGHUP:
		fprintf(stderr, "\n\nCaught SIGHUP..... Cleaning up!\n\n");
		exit(1);
		break;
		case SIGTERM:
		fprintf(stderr, "\n\nCaught SIGTERM..... Cleaning up!\n\n");
		exit(1);
		break;
		case SIGSEGV:
		fprintf(stderr, "Caught SIGSEGV..... Killing thread!\n");
		break;
		case SIGALRM:
		fprintf(stderr, "\n\nCaught SIGALRM..... Cleaning up!\n\n");
		return;
		break;
	}
	fflush(stdout);
	fflush(stderr);
	o.cur_childs--;
	pthread_detach(pthread_self());
	pthread_exit(NULL);
	
}

void sig_int2(int signo) {


	fprintf(stderr, "Caught SIGPIPE...... Trying to continue!\n");

	//pthread_detach(pthread_self());
	//pthread_exit(NULL);

}


void connect_alarm(int signo)
{

	fprintf(stderr, "\nGot sigalarm timeout!\n\n");
	return;
}

void *s_malloc(int size)
{
	void *mymem;
	if (size < 0) {
		fprintf(stderr, "\n\nERROR\n\nTried to malloc negative amount of memory!!!\n\n");
		exit(1);
	}
	mymem = malloc(size);
	if (mymem == NULL) {
		fprintf(stderr, "\n\nMalloc Failed. Probably out of memory..... quitting!\n\n");
		exit(1);
	}

	return mymem;
}
	







