/***************************************************************************
                          parse_args2.c  -  description
                             -------------------
    begin                : Tue Apr 16 2002
    copyright            : (C) 2002 by Spencer Hardy
    email                : diceman@dircon.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "relay.h"

char *cpy_opt(char *optarg) {

	char buf[MAXDATA+1];
	int size = 0;
	char *string;

	
	strncpy(buf, optarg, MAXDATA);
	buf[MAXDATA] = '\0';
	size = strlen(buf);
	string = s_malloc((size+1) * sizeof(char));
	strncpy(string, buf, size);
	string[size] = '\0';
	
	return(string);

}

int parse_args(int argc, char * const *argv) {


	char c;

	while((c = getopt(argc, argv, "ab:c:de:f:g:hi:j:k:l:m:no:p:r:s:tvw:y:z"))!=-1) {


	
		switch(c) {
		case 'a':		//display all results even those that pass
			o.display++;
			break;
		case 'b':  //email addy to use to replace --NAME-- --DOMAIN--
			f.send_email = 1;
			o.email_addy=cpy_opt(optarg);
			o.email_addy2=cpy_opt(optarg);
			break;
		case 'c':  //Specify the config file to use
			f.use_conf = 1;
			o.config_file=cpy_opt(optarg);
			break;
		case 'd':  //Switch on DEBUG MODE
			o.debug++;
			break;
		case 'e':  //Specify a local mailbox
			f.check_mail = 1;
			o.mail_file=cpy_opt(optarg);
			break;
		case 'f':	//Txtx file to output to			
			f.txt_file = 1;
			o.txt_file=cpy_opt(optarg);
			break;
		case 'g': //set a timeout near threads **DEAD**
			o.goat = atoi(optarg);
			break;
		case 'h': //Display the help file
			usage(argv[0]);
			break;
		case 'i':	//specify an ip list				
			f.ip_list = 1;
			o.ip_list=cpy_opt(optarg);
			break;
		case 'j': //specify an auto config file
			f.aconfig = 1;
			o.aconfig=cpy_opt(optarg);
			break;
		case 'k': //generate a config file
			f.gen_config = 1;
			o.gen_config=cpy_opt(optarg);
      break;
		case 'l': //READ TIMEOUT
			o.c_timeout = atoi(optarg);
			break;
		case 'm': //TIME OUT before checking mail file
			o.email_timeout = atoi(optarg);
			break;
		case 'n': //Resolve hostnames
			f.get_hostname++;
			break;
		case 'o': //PRINT OUTPUT IN MACHINE READABLE FORMAT
			f.print_machine++;
			o.machine_file=cpy_opt(optarg);
			break;
		case 'p': //NUMBER OF THREADS
			o.child = atoi(optarg);
			break;
		case 'r': //READ TIMEOUT
			o.r_timeout = atoi(optarg);
			break;
		case 's': //IP RANGE TO SCAN
			f.ip_range = 1;
			o.ip_range=cpy_opt(optarg);
			o.ip_range2=cpy_opt(optarg);
			break;
		case 't':  //PRINT TO SCREEN **DEAD**
			fprintf(stderr, "\n\n*** This option (-t) doesn't work much it's just for debugging\n\n");
			f.screen++;
			break;
		case 'v':		//be verbose		
			o.verbose++;
			break;
		case 'w':  //output to html file
			f.m_html_file = 1;
			o.m_html_file=cpy_opt(optarg);
			break;
		case 'y':  //specify email file
			f.got_email = 1;
			o.email_file=cpy_opt(optarg);
			break;
		case 'z':	//Verbose html output
			f.verbose_html = 1;
			break;
		case'?':
			usage(argv[0]);
		default:
			usage(argv[0]);
		}		
	}
	
	get_rand();

	if(f.aconfig) {
		parse_aconfig();
	}

	check_cons(argv[0]);

	if(f.ip_range) {
		s_ip_range(o.ip_range);
	}

	if(f.ip_list) {
		parse_ip_list(o.ip_list);
	}
	//parseconfig(o.config_file);
	if(f.send_email) {
		get_domain();
	}

	setup_rules(o.config_file);
	
	//correct_config();

	if(f.got_email) {
    get_email_file(o.email_file);
  }

	
	if(!o.r_timeout) {
		o.r_timeout = RTIMEO;
	}
	if(!o.c_timeout) {
  	o.c_timeout = CIMEO;
	}
	
	return(0);


}

int check_cons(char *pname) {

	FILE *fd;

	char c;

	if(f.ip_list && f.ip_range) {
		fprintf(stderr, "\n\nYou must specify either an ip list or an ip range to scan... not both....... Exiting\n\n");
		exit(1);
	}

	if(!f.ip_list && !f.ip_range) {
		fprintf(stderr, "\n\nYou must specify either an ip list or an ip range to scan...... Exiting\n\n");
		exit(1);
	}

	if(f.check_mail) {
	  	if((fd=fopen(o.mail_file, "r")) == NULL) {
			fprintf(stderr, "\n\nUnable to open mail file %s\n\nContinue anyway? [y]es [n]o\t:", o.mail_file);
			c=fgetc(stdin);
			if(c == 'n' || c == 'N') {
				fprintf(stderr, "\n\nOK.... Exiting\n\n");
				exit(1);
			}
			fprintf(stderr, "\nContinuing..............\n\n");
		}
		else {
			fclose(fd);
		}
	}

	if(!f.use_conf) {
		fprintf(stderr, "\n\nYou must specify a config file with the -c option.....\nUse -h for help\n\n");
		exit(1);
	}

	if(f.print_machine) {	
		if((fd=fopen(o.machine_file, "w")) == NULL) {
			fprintf(stderr, "\n\nERROR\n\nUnable to open the file specified with the -o option\n\n");
			exit(1);
                }
		fclose(fd);
        }

	if(o.child<2) {
		fprintf(stderr, "\n\nSingle thread scanning has not been implemented\n\nYou must specify a number of threads with the -p option\n\nUse the -h option for help\n\n");	
		exit(1);
	}

	return(0);

}
